<!-- begin "Header.html" -->

{% comment %}
{% block TopBanner %}
<div class="container">
{% include "TopBanner.html" %}
{% include "TopRibbon.html" %}
</div><!-- close container for the Header content -->
{% endblock %}

{% endcomment %}
<!-- end "Header.html" -->
