{%  load settings_tags %}
    <div class="topbar">
      <div class="fill">
        <div class="container">
          <a class="brand" href="#">{% settings_value "SITE_NAME" %}</a>
          <ul class="nav">
            <li class="active"><a href="#">Home</a></li>
            <li><a href="#about">About</a></li>
            <li><a href="#contact">Contact</a></li>
          </ul>
        </div>
      </div>
    </div>
