{% load mezzanine_tags pages_tags settings_tags %}
<div class="container" style="margin-top:5px;background-color:#fffff;">
<div class="row">
<div class="navbar navbar-inverse">
<div class="navbar-inner">
    <a class="btn btn-navbar" data-toggle="collapse" data-target=".nav-collapse">
        <span class="icon-bar"></span>
        <span class="icon-bar"></span>
        <span class="icon-bar"></span>
    </a>
    {% if settings.SITE_TITLE %}<a class="brand" href="/">{{ settings.SITE_TITLE }}</a>{% endif %}
    {% if settings.SITE_TAGLINE %}<p class="tagline" style="color:#bbbbbb;font-size:1em;margin-top:1em;">{{ settings.SITE_TAGLINE }}</p>{% endif %}
    <div class="nav-collapse">
         {# {% search_form "all" %} #}
         {% page_menu "pages/menus/dropdown.html" %}
        
    </div>
</div><!-- navbar-inner -->
</div><!-- navbar-inverse -->
</div><!-- row -->
</div><!-- container -->

