{# A simple sidebar menu #}
      <div class="sidebar" style="background:dff;">
        <div class="well" style="background:#ddf;">
	  <h4>Sample SideBar.html</h4>
          <h5>Sidebar</h5>
          <ul>
            <li><a href="/blog/">Blog</a></li>
          </ul>
          <h5>Sidebar</h5>
          <ul>
            <li><a href="#">Link</a></li>
          </ul>
          <h5>Sidebar</h5>
          <ul>
            <li><a href="#">Link</a></li>
            <li><a href="#">Link</a></li>
          </ul>
        </div>
      </div>
