from distutils.core import setup

setup(
    name='backstage',
    version='0.0.01',
    packages=[''],
    package_dir={'': '.idea'},
    url='',
    license='',
    author='walker',
    author_email='',
    description='', requires=['django']
)
