{% load settings_tags %}
        <!-- Main hero unit for a primary marketing message or call to action -->
        <div class="hero-unit">
          <h1>{% settings_value "PRETTYNAME" %}</h1>
          <p>This is your Main blurb or pitch or call to action.   It's named 'HeroUnit.html'   The default version is located in the project templates file. You can be copy into your site's template folder and customize it from there, and the changes will appear here.</p>
          <p><a class="btn primary large">Learn more &raquo;</a></p>
        </div>
