    <div class="container">

      <div class="content">
        <div class="page-header">
          <h1>Page name <small>Supporting text or tagline</small></h1>
        </div>
        <div class="row">
          <div class="col-lg-10 col-md-10 col-sm-10">
            <h2>Main content</h2>
          </div>
          <div class="col-lg-4 col-md-4 col-sm-4">
            <h3>Secondary content</h3>
          </div>
        </div>
      </div>

      <footer>
        <p>&copy; Company 2011</p>
      </footer>

    </div> <!-- /container -->
